<?php
/**
 * Loulou66
 * LpsHomeTabCarousel module for Prestashop
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php*
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    Loulou66.fr <contact@loulou66.fr>
 *  @copyright loulou66.fr
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class LpsHomeTabCarouselClass extends ObjectModel
{
    public $type;

    public $prod_category;

    public $prod_display;

    public $prod_visible;

    public $loop;

    public $autoplay;

    public $random;

    public $active;

    public $position;

    public $id_shop;

    public $title;

    public static $definition = [
        'table' => 'lpshometabcarousel',
        'primary' => 'id_lpshometabcarousel',
        'multilang' => true,
        'multilang_shop' => false,
        'fields' => [
            'type' => ['type' => self::TYPE_STRING],
            'prod_category' => ['type' => self::TYPE_INT, 'validate' => 'isUnsignedInt'],
            'prod_display' => ['type' => self::TYPE_INT, 'validate' => 'isUnsignedInt'],
            'prod_visible' => ['type' => self::TYPE_INT, 'validate' => 'isUnsignedInt'],
            'loop' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool'],
            'autoplay' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool'],
            'random' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool'],
            'active' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool'],
            'position' => ['type' => self::TYPE_INT, 'validate' => 'isUnsignedInt'],
            'id_shop' => ['type' => self::TYPE_INT, 'validate' => 'isUnsignedId'],
            'title' => [
                'type' => self::TYPE_STRING,
                'lang' => true,
                'validate' => 'isCatalogName',
                'size' => 128,
            ],
        ],
    ];

    public function add($autoDate = true, $nullValues = false)
    {
        if ($this->position <= 0) {
            $this->position = LpsHomeTabCarouselClass::getHigherPosition() + 1;
        }
        $this->id_shop = (int) Context::getContext()->shop->id;
        $autoDate = false;
        $return = parent::add($autoDate, $nullValues);
        return $return;
    }

    public static function getHigherPosition()
    {
        $sql = 'SELECT MAX(position) FROM `' . _DB_PREFIX_ . 'lpshometabcarousel`';
        $position = Db::getInstance()->getValue($sql);
        return is_numeric($position) ? $position : -1;
    }

    public function copyFromPost()
    {
        foreach ($_POST as $key => $value) {
            if (array_key_exists($key, get_object_vars($this)) and $key != 'id_' . $this->table) {
                $this->{$key} = $value;
            }
        }
        if (sizeof($this->fieldsValidateLang)) {
            $languages = Language::getLanguages(false);
            foreach ($languages as $lang) {
                foreach ($this->fieldsValidateLang as $field => $validation) {
                    $validation = $field;
                    if (Tools::isSubmit($field . '_' . (int) $lang['id_lang'])) {
                        $this->{$field}[(int) $lang['id_lang']] = Tools::getValue(
                            $field . '_' . (int) $lang['id_lang']
                        );
                    }
                }
            }
        }
    }

    public static function updatePositions()
    {
        $positions = Tools::getValue('module-lpshometabcarousel');
        $new_positions = [];
        foreach ($positions as $v) {
            if (count(explode('_', $v)) == 4) {
                $new_positions[] = $v;
            }
        }
        foreach ($new_positions as $position => $value) {
            $pos = explode('_', $value);
            $lpshometabcarousel = new LpsHomeTabCarouselClass((int) $pos[2]);
            $lpshometabcarousel->position = $position;
            $lpshometabcarousel->save();
        }
    }

    public static function updatePositionsAfterDelete($id_lang, $id_shop)
    {
        $lpshometabcarousel = self::getAllCarousel((int) $id_lang, (int) $id_shop);
        $position = 0;
        foreach ($lpshometabcarousel as $carousel) {
            $lpsHTC = new LpsHomeTabCarouselClass((int) $carousel['id_lpshometabcarousel']);
            $lpsHTC->position = $position;
            $lpsHTC->save();
            ++$position;
        }
    }

    public static function statusToggle($id_lpshometabcarousel)
    {
        $lpsHTC = new LpsHomeTabCarouselClass((int) $id_lpshometabcarousel);
        if (Validate::isLoadedObject($lpsHTC)) {
            $lpsHTC->active = !$lpsHTC->active;
            return $lpsHTC->update();
        } else {
            return false;
        }
    }

    public static function getAllCarousel($id_lang, $id_shop)
    {
        $sql = 'SELECT lpshtc.*,  lpshtcl.*
            FROM `' . _DB_PREFIX_ . 'lpshometabcarousel` lpshtc
            LEFT JOIN `' . _DB_PREFIX_ . 'lpshometabcarousel_lang` lpshtcl
            ON (lpshtc.`id_lpshometabcarousel`= lpshtcl.`id_lpshometabcarousel`)
            WHERE lpshtc.`id_shop`=' . (int) $id_shop . '
            AND lpshtcl.`id_lang`=' . (int) $id_lang . '
            AND lpshtc.`active`= 1
            ORDER BY lpshtc.`position` ASC';
        return Db::getInstance()->executeS($sql);
    }

    public static function getNewProducts($id_shop, $id_lang, $nb_products = 10)
    {
        $front = true;
        $page_number = 1;
        $order_by = 'date_add';
        $order_way = 'DESC';
        $order_by_prefix = 'product_shop';
        $nb_days_new_product = (int) Configuration::get('PS_NB_DAYS_NEW_PRODUCT')
            ? (int) Configuration::get('PS_NB_DAYS_NEW_PRODUCT')
            : 20;
        $sql = new DbQuery();
        $sql->select(
            'p.*, product_shop.*, stock.out_of_stock, IFNULL(stock.quantity, 0) as quantity, pl.`description`,
            pl.`description_short`, pl.`link_rewrite`, pl.`meta_description`, pl.`meta_keywords`, pl.`meta_title`,
            pl.`name`, pl.`available_now`, pl.`available_later`, image_shop.`id_image` id_image, il.`legend`,
            m.`name` AS manufacturer_name,
            product_shop.`date_add` > "' . date('Y-m-d', strtotime('-' . $nb_days_new_product . ' DAY')) . '" as new'
        );
        $sql->from('product', 'p');
        $sql->join(Shop::addSqlAssociation('product', 'p'));
        $sql->leftJoin(
            'product_lang',
            'pl',
            'p.`id_product` = pl.`id_product` AND pl.`id_lang` = ' . (int) $id_lang .
                Shop::addSqlRestrictionOnLang('pl')
        );
        $sql->leftJoin(
            'image_shop',
            'image_shop',
            'image_shop.`id_product` = p.`id_product` AND image_shop.cover=1 AND image_shop.id_shop=' . (int) $id_shop
        );
        $sql->leftJoin(
            'image_lang',
            'il',
            'image_shop.`id_image` = il.`id_image` AND il.`id_lang` = ' . (int) $id_lang
        );
        $sql->leftJoin('manufacturer', 'm', 'm.`id_manufacturer` = p.`id_manufacturer`');
        $sql->where('product_shop.`active` = 1');
        if ($front) {
            $sql->where('product_shop.`visibility` IN ("both", "catalog")');
        }
        $sql->where(
            'product_shop.`date_add` > "' . date('Y-m-d', strtotime('-' . $nb_days_new_product . ' DAY')) . '"'
        );
        if (Group::isFeatureActive()) {
            $groups = FrontController::getCurrentCustomerGroups();
            $sqlgroup = count($groups) ? 'IN (' . implode(',', $groups) . ')' : '=' . (int) Group::getCurrent()->id;
            $sqlWhere = 'EXISTS(
                SELECT 1
                FROM `' . _DB_PREFIX_ . 'category_product` cp
                JOIN `' . _DB_PREFIX_ . 'category_group` cg ON (cp.`id_category` = cg.`id_category`)
                WHERE cp.`id_product` = p.`id_product`
                AND cg.`id_group` ' . $sqlgroup .
            ')';
            $sql->where($sqlWhere);
        }
        $sql->orderBy(pSQL($order_by_prefix) . '.`' . pSQL($order_by) . '` ' . pSQL($order_way));
        $sql->limit($nb_products, (int) (($page_number - 1) * $nb_products));
        if (Combination::isFeatureActive()) {
            $sql->select('product_attribute_shop.minimal_quantity AS product_attribute_minimal_quantity, IFNULL(product_attribute_shop.id_product_attribute,0) id_product_attribute');
            $sql->leftJoin('product_attribute_shop', 'product_attribute_shop', 'p.`id_product` = product_attribute_shop.`id_product` AND product_attribute_shop.`default_on` = 1 AND product_attribute_shop.id_shop=' . (int) $id_shop);
        }
        $sql->join(Product::sqlStock('p', 0));
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
        if (!$result) {
            return false;
        }
        return Product::getProductsProperties((int) $id_lang, $result);
    }
}
