<?php
/**
 * Loulou66
 * LpsHomeTabCarousel module for Prestashop
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php*
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    Loulou66.fr <contact@loulou66.fr>
 *  @copyright loulou66.fr
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
use PrestaShop\PrestaShop\Adapter\BestSales\BestSalesProductSearchProvider;
use PrestaShop\PrestaShop\Adapter\Category\CategoryProductSearchProvider;
use PrestaShop\PrestaShop\Adapter\Image\ImageRetriever;
use PrestaShop\PrestaShop\Adapter\Product\PriceFormatter;
use PrestaShop\PrestaShop\Adapter\Product\ProductColorsRetriever;
use PrestaShop\PrestaShop\Core\Product\ProductListingPresenter;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchContext;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchQuery;
use PrestaShop\PrestaShop\Core\Product\Search\SortOrder;

if (!defined('_PS_VERSION_')) {
    exit;
}

require_once _PS_MODULE_DIR_ . 'lpshometabcarousel/classes/lpshometabcarouselclass.php';

class LpsHomeTabCarousel extends Module
{
    public $html;

    public $confirmation;

    public $errors = [];

    public $templateFile;

    public $template_content;

    public $list_types;

    public $list_id;

    public $_default_pagination;

    public $_pagination;

    public $orderby;

    public $fields_list;

    public $fields_Form;

    public $fields_value;

    public function __construct()
    {
        $this->name = 'lpshometabcarousel';
        $this->author = 'Loulou66';
        $this->version = '8.0.2';
        $this->tab = 'front_office_features';
        $this->need_instance = 0;
        $this->ps_versions_compliancy = ['min' => '1.7.0', 'max' => _PS_VERSION_];
        $this->bootstrap = true;
        parent::__construct();
        $this->displayName = $this->l('LPS Homepage Tabs with Carousel');
        $this->description = $this->l('Boost your homepage by displaying your products in clickable CAROUSEL TABS');
        $this->templateFile = 'module:lpshometabcarousel/views/templates/hook/lpshometabcarousel.tpl';
        $this->template_content = 'module:lpshometabcarousel/views/templates/hook/lpshometabcarousel_content.tpl';
        $this->module_key = '25692b51e1a7ca0228c42d2d99840af9';
        $this->list_types = [
            'featured' => $this->l('Featured Products'),
            'new' => $this->l('New Products'),
            'best' => $this->l('Best Sellers'),
            'specials' => $this->l('Specials'),
        ];
    }

    public function install()
    {
        $this->_clearCache('*');
        include dirname(__FILE__) . '/sql/install.php';
        return parent::install()
            && $this->registerHook('actionAdminControllerSetMedia')
            && $this->registerHook('actionFrontControllerSetMedia')
            && $this->registerHook('actionOrderStatusPostUpdate')
            && $this->registerHook('actionProductAdd')
            && $this->registerHook('actionProductUpdate')
            && $this->registerHook('actionProductDelete')
            && $this->registerHook('displayHome')
            && ProductSale::fillProductSales();
    }

    public function uninstall()
    {
        $this->_clearCache('*');
        include dirname(__FILE__) . '/sql/uninstall.php';
        return parent::uninstall();
    }

    public function isUsingNewTranslationSystem()
    {
        return false;
    }

    public function hookActionProductAdd($params)
    {
        $this->_clearCache('*');
    }

    public function hookActionProductUpdate($params)
    {
        $this->_clearCache('*');
    }

    public function hookActionProductDelete($params)
    {
        $this->_clearCache('*');
    }

    public function hookActionOrderStatusPostUpdate($params)
    {
        $this->_clearCache('*');
    }

    public function hookActionAdminControllerSetMedia($params)
    {
        if ($this->context->controller->controller_name == 'AdminModules'
            && Tools::getValue('configure') == $this->name
        ) {
            $this->context->controller->addCSS($this->_path . 'views/css/' . $this->name . 'admin.css');
            $this->context->controller->addJS($this->_path . 'views/js/' . $this->name . 'admin.js');
        }
    }

    public function getWarningMultishopHtml()
    {
        return $this->displayWarning(
            sprintf(
                $this->l('You cannot manage %s from a "All Shops" or a "Group Shop" context.'),
                $this->displayName
            ) .
            ' ' .
            $this->l('Select directly the shop you want to edit.')
        );
    }

    public function getContent()
    {
        $this->html = '';
        $id_lpshometabcarousel = Tools::getValue('id_lpshometabcarousel');
        if (Shop::isFeatureActive()) {
            if (Shop::getContext() == Shop::CONTEXT_GROUP || Shop::getContext() == Shop::CONTEXT_ALL) {
                return $this->getWarningMultishopHtml();
            }
        }
        if (Tools::isSubmit('add' . $this->name) || Tools::isSubmit('update' . $this->name)) {
            return $this->renderForm();
        }
        if (Tools::isSubmit('submit' . $this->name)) {
            $this->postProcess();
            if (!count($this->errors)) {
                if ($id_lpshometabcarousel) {
                    $lpsHTC = new LpsHomeTabCarouselClass((int) $id_lpshometabcarousel);
                } else {
                    $lpsHTC = new LpsHomeTabCarouselClass();
                }
                $lpsHTC->copyFromPost();
                $def_lang = Configuration::get('PS_LANG_DEFAULT');
                foreach (Language::getLanguages(true, (int) $this->context->shop->id) as $lang) {
                    if (empty(Tools::getValue('title_' . $lang['id_lang']))) {
                        $lpsHTC->title[$lang['id_lang']] = $lpsHTC->title[$def_lang];
                    }
                }
                $lpsHTC->id_shop = $this->context->shop->id;
                if (!$lpsHTC->save()) {
                    $this->errors[] = $this->displayError($this->l('Unable to save carousel'));
                }
            }
            if (count($this->errors)) {
                foreach ($this->errors as $error) {
                    $this->html .= $error;
                }
                return $this->html . $this->renderForm();
            } else {
                if ($id_lpshometabcarousel) {
                    $this->html .= $this->displayConfirmation($this->l('Carousel updated.'));
                } else {
                    $this->html .= $this->displayConfirmation($this->l('Carousel created Successfully.'));
                }
            }
        }
        if (Tools::isSubmit('updatePositions') && Tools::getValue('configure') == $this->name) {
            LpsHomeTabCarouselClass::updatePositions();
        }
        if (Tools::isSubmit('status' . $this->name)) {
            $id_lpshometabcarousel = Tools::getValue('id_lpshometabcarousel');
            if (!LpsHomeTabCarouselClass::statusToggle(Tools::getValue('id_lpshometabcarousel'))) {
                $this->html .= $this->displayError($this->l('Unable to update status'));
            } else {
                $this->html .= $this->displayConfirmation($this->l('The status has been updated'));
            }
        }
        if (Tools::isSubmit('delete' . $this->name)) {
            $id_lpshometabcarousel = Tools::getValue('id_lpshometabcarousel');
            $lpsHTC = new LpsHomeTabCarouselClass((int) $id_lpshometabcarousel);
            if (!$lpsHTC->delete()) {
                $this->html .= $this->displayError($this->l('Unable to delete the carousel'));
            } else {
                LpsHomeTabCarouselClass::updatePositionsAfterDelete(
                    (int) $this->context->language->id,
                    (int) $this->context->shop->id
                );
                $this->html .= $this->displayConfirmation($this->l('Successful deletion.'));
            }
        }
        if (Tools::isSubmit('submitReset' . $this->name)) {
            $this->context->controller->processResetFilters($this->name);
        }
        if (Tools::isSubmit('submitBulkdelete' . $this->name)) {
            $boxSelected = Tools::getValue($this->name . 'Box');
            $res = true;
            foreach ($boxSelected as $id) {
                $lpsRFtemp = new LpsHomeTabCarouselClass((int) $id);
                $res &= $lpsRFtemp->delete();
            }
            if ($res) {
                LpsHomeTabCarouselClass::updatePositionsAfterDelete(
                    (int) $this->context->language->id,
                    (int) $this->context->shop->id
                );
                $this->html .= $this->displayConfirmation($this->l('The selection has been successfully deleted.'));
            }
        }
        return $this->html . $this->initList();
    }

    public function postProcess()
    {
        $deflang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));
        if (empty(Tools::getValue('title_' . Configuration::get('PS_LANG_DEFAULT')))) {
            $this->errors[] = $this->displayError(
                sprintf($this->l('The « title » field is required at least in %s.'), $deflang->name)
            );
        }
        foreach (Language::getLanguages(true, (int) $this->context->shop->id) as $lang) {
            if (!Validate::isCatalogName(Tools::getValue('title_' . $lang['id_lang']))) {
                $this->errors[] = $this->displayError(
                    sprintf($this->l('The « title » field (%s) is invalid.'), $lang['name'])
                );
            } elseif (Tools::strlen(Tools::getValue('title_' . $lang['id_lang'])) > 128) {
                $this->errors[] = $this->displayError(
                    sprintf(
                        $this->l(
                            'The value of « title » field is too long in %s. It should have 128 character or less.'
                        ),
                        $lang['name']
                    )
                );
            }
        }
        if (!Validate::isInt((int) Tools::getValue('prod_display')) || (int) Tools::getValue('prod_display') <= 0) {
            $this->errors[] = $this->displayError(
                $this->l('The « Products to display » field is invalid. Please enter a positive number.')
            );
        }
    }

    public function initList()
    {
        $this->list_id = 'lpshometabcarousel';
        $this->_default_pagination = '50';
        $this->_pagination = ['50'];
        $this->orderby = Tools::getValue($this->list_id . 'Orderby')
            ? Tools::getValue($this->list_id . 'Orderby')
            : 'position';
        $this->fields_list = [
            'id_lpshometabcarousel' => [
                'title' => $this->l('ID'),
                'type' => 'text',
                'orderby' => true,
                'search' => true,
            ],
            'title' => [
                'title' => $this->l('Title'),
                'type' => 'text',
                'lang' => true,
                'orderby' => true,
                'search' => true,
            ],
            'type' => [
                'title' => $this->l('Type'),
                'type' => 'select',
                'list' => $this->list_types,
                'filter_key' => 'type',
                'orderby' => true,
                'search' => true,
            ],
            'position' => [
                'title' => $this->l('Position'),
                'position' => 'position',
                'align' => 'center',
                'orderby' => false,
                'search' => false,
                'class' => 'fixed-width-xs',
            ],
            'active' => [
                'title' => $this->l('Active'),
                'active' => 'status',
                'type' => 'bool',
                'align' => 'text-center',
                'orderby' => false,
                'search' => false,
            ],
        ];
        $helperList = new HelperList();
        $helperList->simple_header = false;
        $helperList->name_controller = $this->name;
        $helperList->table_id = 'module-' . $this->name;
        $helperList->table = $this->list_id;
        $helperList->identifier = 'id_' . $this->list_id;
        $helperList->_default_pagination = $this->_default_pagination;
        $helperList->_pagination = $this->_pagination;
        $helperList->orderBy = $this->orderby;
        $helperList->orderWay = 'ASC';
        $helperList->position_identifier = 'position';
        $helperList->position_group_identifier = 0;
        $helperList->module = $this;
        $helperList->actions = ['edit', 'delete'];
        $helperList->no_link = true;
        if (Shop::isFeatureActive() &&
            (Shop::getContext() != Shop::CONTEXT_SHOP || Shop::getContext() != Shop::CONTEXT_ALL)) {
            $helperList->shopLinkType = true;
        } else {
            $helperList->shopLinkType = false;
        }
        $helperList->title = $this->l('Homepage Carousels');
        $helperList->token = Tools::getAdminTokenLite('AdminModules');
        $helperList->show_toolbar = 1;
        $helperList->currentIndex = AdminController::$currentIndex .
            '&configure=' . $this->name .
            '&tab_module=' . $this->tab .
            '&module_name=' . $this->name;
        $helperList->toolbar_btn = [
            'new' => [
                'href' => AdminController::$currentIndex .
                    '&configure=' . $this->name .
                    '&tab_module=' . $this->tab .
                    '&module_name=' . $this->name .
                    '&add' . $this->name .
                    '&token=' . Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Add new carousel'),
            ],
        ];
        $helperList->bulk_actions = [
            'delete' => [
                'text' => $this->l('Delete selected'),
                'icon' => 'icon-trash',
                'confirm' => $this->l('These carousel will be deleted. Please confirm.'),
            ],
        ];
        $helperList->listTotal = count($this->getCustomerFields(true));
        return $helperList->generateList($this->getCustomerFields(false), $this->fields_list);
    }

    public function getCustomerFields($listTotal = false, $start = 0, $limit = null)
    {
        $sql = 'SELECT lpshtc.*,  lpshtcl.*
            FROM `' . _DB_PREFIX_ . 'lpshometabcarousel` lpshtc
            LEFT JOIN `' . _DB_PREFIX_ . 'lpshometabcarousel_lang` lpshtcl
            ON (lpshtc.`id_lpshometabcarousel`= lpshtcl.`id_lpshometabcarousel`)
            WHERE lpshtc.`id_shop`=' . (int) $this->context->shop->id . '
            AND lpshtcl.`id_lang`=' . (int) $this->context->language->id;
        if (Tools::getValue('submitFilter' . $this->list_id)) {
            foreach ($this->fields_list as $f => $field) {
                $filter = Tools::getValue($this->list_id . 'Filter_' . $f);
                $myfield = $field;
                if (!empty($filter)) {
                    $sql .= ' AND ';
                    $sql .= (isset($field['lang']) && $field['lang']) ? 'lpshtcl.`' . $f . '`' : 'lpshtc.`' . $f . '`';
                    $sql .= ' LIKE "%' . $filter . '%"';
                }
                unset($myfield);
            }
        }
        if ($orderby = Tools::getValue($this->list_id . 'Orderby')) {
            $alias = ($orderby == 'title') ? 'lpshtcl.`' : 'lpshtc.`';
            $sql .= ' ORDER BY ' . $alias . $orderby . '` ' . Tools::getValue($this->list_id . 'Orderway');
            $this->context->cookie->{$this->list_id . 'Orderby'} = $orderby;
            $this->context->cookie->{$this->list_id . 'Orderway'} = Tools::getValue($this->list_id . 'Orderway');
        } else {
            $sql .= ' ORDER BY lpshtc.`position` ASC';
            $this->context->cookie->{$this->list_id . 'Orderby'} = 'position';
            $this->context->cookie->{$this->list_id . 'Orderway'} = 'ASC';
        }
        if (empty($limit)) {
            if (isset($this->context->cookie->{$this->list_id . '_pagination'}) &&
                $this->context->cookie->{$this->list_id . '_pagination'}) {
                $limit = $this->context->cookie->{$this->list_id . '_pagination'};
            } else {
                $limit = $this->_default_pagination;
            }
        }
        $limit = (int) Tools::getValue($this->list_id . '_pagination', $limit);
        if (in_array($limit, $this->_pagination) && $limit != $this->_default_pagination) {
            $this->context->cookie->{$this->list_id . '_pagination'} = $limit;
        } else {
            unset($this->context->cookie->{$this->list_id . '_pagination'});
        }
        if ((int) Tools::getValue('submitFilter' . $this->list_id)) {
            $start = ((int) Tools::getValue('submitFilter' . $this->list_id) - 1) * $limit;
        } elseif (empty($start) && isset($this->context->cookie->{$this->list_id . '_start'})) {
            $start = $this->context->cookie->{$this->list_id . '_start'};
        }
        if ($start) {
            $this->context->cookie->{$this->list_id . '_start'} = $start;
        } elseif (isset($this->context->cookie->{$this->list_id . '_start'})) {
            unset($this->context->cookie->{$this->list_id . '_start'});
        }
        if (!$listTotal) {
            $sql .= ' LIMIT ' . (int) $start . ', ' . (int) $limit;
        }
        $list = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
        foreach ($list as &$row) {
            $row['type'] = $this->list_types[$row['type']];
            if (Shop::isFeatureActive()) {
                $row['shop_name'] = $this->context->shop->name;
            }
        }
        return $list;
    }

    public function renderForm()
    {
        $list_types = [
            ['id' => 'featured', 'name' => $this->l('Featured Products')],
            ['id' => 'new', 'name' => $this->l('New Products')],
            ['id' => 'best', 'name' => $this->l('Best Sellers')],
            ['id' => 'specials', 'name' => $this->l('Specials')],
        ];
        $selected_categories = 2;
        if ($id_lpshometabcarousel = Tools::getValue('id_lpshometabcarousel')) {
            $lpsHTC = new LpsHomeTabCarouselClass((int) $id_lpshometabcarousel);
            $selected_categories = $lpsHTC->prod_category;
        }
        $this->fields_Form = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Carousel Settings'),
                    'icon' => 'icon-cogs',
                ],
                'input' => [
                    [
                        'type' => 'hidden',
                        'name' => 'id_lpshometabcarousel',
                        'default_value' => '',
                    ],
                    [
                        'type' => 'hidden',
                        'name' => 'position',
                        'default_value' => '',
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Title'),
                        'name' => 'title',
                        'lang' => true,
                        'desc' => $this->l('Invalid characters:') . ' <>{}.',
                        'hint' => $this->l('Invalid characters:') . ' <>{}.',
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->l('Type'),
                        'name' => 'type',
                        'desc' => $this->l('Select carousel type'),
                        'hint' => $this->l('Select carousel type'),
                        'default_value' => 'text',
                        'options' => [
                            'query' => $list_types,
                            'id' => 'id',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'type' => 'categories',
                        'label' => $this->l('Category'),
                        'tree' => [
                          'id' => 'home_featured_category',
                          'selected_categories' => [$selected_categories],
                        ],
                        'name' => 'prod_category',
                        'desc' => $this->l('Select the category of products to display'),
                        'hint' => $this->l('Select the category of products to display'),
                        'form_group_class' => 'lpsprodcategory',
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Products to display'),
                        'name' => 'prod_display',
                        'desc' => $this->l('Determine the total number of products to display in the carousel'),
                        'hint' => $this->l('Determine the total number of products to display in the carousel'),
                        'default_value' => 20,
                    ],
                    [
                        'type' => 'html',
                        'label' => $this->l('Products visible'),
                        'html_content' => $this->displayProdVisible(),
                        'name' => '',
                        'desc' => $this->l('Define the number of products visible'),
                        'hint' => $this->l('Define the number of products visible'),
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Loop'),
                        'name' => 'loop',
                        'is_bool' => true,
                        'default_value' => true,
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Autoplay'),
                        'name' => 'autoplay',
                        'is_bool' => true,
                        'default_value' => true,
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Random display'),
                        'name' => 'random',
                        'hint' => $this->l('Enable if you wish the products to be displayed randomly'),
                        'desc' => $this->l('Enable if you wish the products to be displayed randomly'),
                        'is_bool' => true,
                        'default_value' => false,
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Enable Carousel'),
                        'name' => 'active',
                        'is_bool' => true,
                        'default_value' => true,
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                    'name' => 'save' . $this->name,
                    'class' => 'button btn btn-default',
                    'icon' => 'process-icon-save',
                ],
                'buttons' => [
                    [
                        'id' => $this->name . '_form_cancel_btn',
                        'href' => AdminController::$currentIndex .
                            '&configure=' . $this->name .
                            '&tab_module=' . $this->tab .
                            '&module_name=' . $this->name .
                            '&token=' . Tools::getAdminTokenLite('AdminModules'),
                        'title' => $this->l('Cancel'),
                        'icon' => 'process-icon-cancel',
                    ],
                ],
            ],
        ];
        $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG')
            ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG')
            : 0;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submit' . $this->name;
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) .
            '&configure=' . $this->name .
            '&tab_module=' . $this->tab .
            '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = [
            'fields_value' => $this->getCarouselFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        ];
        return $helper->generateForm([$this->fields_Form]);
    }
    public function displayProdVisible()
    {
        $tpl = _PS_MODULE_DIR_ . $this->name . '/views/templates/admin/lpsbsprodvisible.tpl';
        $lpsbsprodvisible = $this->context->smarty->createTemplate($tpl);
        $prod_visible = 4;
        if ($id_lpshometabcarousel = Tools::getValue('id_lpshometabcarousel')) {
            $lpsHTC = new LpsHomeTabCarouselClass((int) $id_lpshometabcarousel);
            $prod_visible = $lpsHTC->prod_visible;
        }
        $lpsbsprodvisible->assign(['lpshtc_prod_visible' => (int) $prod_visible]);
        return $lpsbsprodvisible->fetch();
    }
    public function getCarouselFieldsValues()
    {
        $id_lpshometabcarousel = Tools::getValue('id_lpshometabcarousel');
        $object = [];
        if ($id_lpshometabcarousel) {
            $object = new LpsHomeTabCarouselClass((int) $id_lpshometabcarousel);
        }
        foreach ($this->fields_Form as $fieldset) {
            if (isset($fieldset['input'])) {
                foreach ($fieldset['input'] as $input) {
                    if (!isset($this->fields_value[$input['name']])) {
                        if (isset($input['lang']) && $input['lang']) {
                            foreach (Language::getIDs(true, (int) $this->context->shop->id) as $id_lang) {
                                $field_value = $this->getFieldValue($object, $input['name'], $id_lang);
                                $this->fields_value[$input['name']][$id_lang] = $field_value;
                            }
                        } else {
                            $field_value = $this->getFieldValue($object, $input['name']);
                            if ($field_value === false && isset($input['default_value'])) {
                                $field_value = $input['default_value'];
                            }
                            $this->fields_value[$input['name']] = $field_value;
                        }
                    }
                }
            }
        }
        return $this->fields_value;
    }

    public function getFieldValue($object, $key, $id_lang = null)
    {
        if ($id_lang) {
            $default_value = (isset($object->id) && $object->id && isset($object->{$key}[$id_lang]))
                ? $object->{$key}[$id_lang]
                : false;
        } else {
            $default_value = isset($object->{$key}) ? $object->{$key} : false;
        }
        return Tools::getValue($key . ($id_lang ? '_' . $id_lang : ''), $default_value);
    }

    public function hookActionFrontControllerSetMedia()
    {
        if (!$this->active) {
            return false;
        }
        $this->context->controller->registerStylesheet(
            'modules-lpshometabcarouselcss',
            'modules/' . $this->name . '/views/css/lpshometabcarouselfront.css'
        );
        $this->context->controller->registerStylesheet(
            'modules-owlcarouselcss',
            'modules/' . $this->name . '/libs/owlcarousel/css/owl.carousel.css'
        );
        $this->context->controller->registerStylesheet(
            'modules-owlcarouselthemecss',
            'modules/' . $this->name . '/libs/owlcarousel/css/owl.theme.default.css'
        );
        $this->context->controller->registerJavascript(
            'modules-owlcarouseljs',
            'modules/' . $this->name . '/libs/owlcarousel/js/owl.carousel.min.js'
        );
        $this->context->controller->registerJavascript(
            'modules-lpshometabcarouseljs',
            'modules/' . $this->name . '/views/js/lpshometabcarousel.js'
        );
    }

    public function HookDisplayHome()
    {
        if (!$this->active) {
            return false;
        }
        $allCarousel = LpsHomeTabCarouselClass::getAllCarousel(
            (int) $this->context->language->id,
            (int) $this->context->shop->id
        );
        if ($allCarousel) {
            foreach ($allCarousel as &$carousel) {
                $cache_id = 'lpshometabcarousel|' . $carousel['id_lpshometabcarousel'];
                $carousel['slides'] = false;
                if ($carousel['random']) {
                    $products = $this->getProducts($carousel);
                    if (!empty($products)) {
                        $lpshtc_hidenav = false;
                        if (count($products) <= $carousel['prod_visible']) {
                            $lpshtc_hidenav = true;
                        }
                        $carousel['slides'] = $this->context->smarty->createTemplate($this->template_content)
                            ->assign(
                                [
                                    'id_carousel' => $carousel['id_lpshometabcarousel'],
                                    'products' => $products,
                                    'lpshtc_hidenav' => $lpshtc_hidenav,
                                    'loop' => $carousel['loop'],
                                    'autoplay' => $carousel['autoplay'],
                                    'prodvisible' => $carousel['prod_visible'],
                                ]
                            )->fetch();
                    }
                } else {
                    if (!$this->isCached($this->template_content, $cache_id)) {
                        $products = $this->getProducts($carousel);
                        if (!empty($products)) {
                            $lpshtc_hidenav = false;
                            if (count($products) <= $carousel['prod_visible']) {
                                $lpshtc_hidenav = true;
                            }
                            $carousel['slides'] = $this->context->smarty->createTemplate($this->template_content)
                                ->assign(
                                    [
                                        'id_carousel' => $carousel['id_lpshometabcarousel'],
                                        'products' => $products,
                                        'lpshtc_hidenav' => $lpshtc_hidenav,
                                        'loop' => $carousel['loop'],
                                        'autoplay' => $carousel['autoplay'],
                                        'prodvisible' => $carousel['prod_visible'],
                                    ]
                                )->fetch();
                        }
                    } else {
                        $carousel['slides'] = $this->fetch($this->template_content, $cache_id);
                    }
                }
            }
            $this->context->smarty->assign(['lpshomeTabcarousel' => $allCarousel]);
            return $this->fetch($this->templateFile);
        }
    }

    public function getProducts($carousel)
    {
        switch ($carousel['type']) {
            case 'featured':
                return $this->getCatProducts($carousel);
            case 'new':
                return $this->getNewProducts($carousel);
            case 'best':
                return $this->getBestSellers($carousel);
            case 'specials':
                return $this->getSpecials($carousel);
            default:
                break;
        }
    }

    public function getCatProducts($carousel)
    {
        $lpsfeaturedproducts_cat = new Category((int) $carousel['prod_category']);
        $searchProvider = new CategoryProductSearchProvider(
            $this->context->getTranslator(),
            $lpsfeaturedproducts_cat
        );
        $context = new ProductSearchContext($this->context);
        $query = new ProductSearchQuery();
        $nProducts = $carousel['prod_display'];
        if ($nProducts < 0) {
            $nProducts = 20;
        }
        $query->setResultsPerPage($nProducts)->setPage(1);
        if ($carousel['random']) {
            $query->setSortOrder(SortOrder::random());
        } else {
            $query->setSortOrder(new SortOrder('product', 'position', 'asc'));
        }
        $lpsfeaturedproducts_resuslt = $searchProvider->runQuery($context, $query);
        $assembler = new ProductAssembler($this->context);
        $presenterFactory = new ProductPresenterFactory($this->context);
        $presentationSettings = $presenterFactory->getPresentationSettings();
        $presenter = $presenterFactory->getPresenter();
        $products_for_template = [];
        $lpsfeaturedproducts = $lpsfeaturedproducts_resuslt->getProducts();
        foreach ($lpsfeaturedproducts as $rawProduct) {
            $products_for_template[] = $presenter->present(
                $presentationSettings,
                $assembler->assembleProduct($rawProduct),
                $this->context->language
            );
        }
        return $products_for_template;
    }

    public function getNewProducts($carousel)
    {
        $lpsnewproducts = false;
        $nbProducts = $carousel['prod_display'];
        if (Configuration::get('PS_NB_DAYS_NEW_PRODUCT')) {
            $lpsnewproducts = LpsHomeTabCarouselClass::getNewProducts(
                (int) $this->context->shop->id,
                (int) $this->context->language->id,
                $nbProducts
            );
        }
        $assembler = new ProductAssembler($this->context);
        $presenterFactory = new ProductPresenterFactory($this->context);
        $presentationSettings = $presenterFactory->getPresentationSettings();
        $presenter = new ProductListingPresenter(
            new ImageRetriever(
                $this->context->link
            ),
            $this->context->link,
            new PriceFormatter(),
            new ProductColorsRetriever(),
            $this->context->getTranslator()
        );
        $products_for_template = [];
        if (is_array($lpsnewproducts)) {
            if ($carousel['random']) {
                shuffle($lpsnewproducts);
            }
            foreach ($lpsnewproducts as $rawProduct) {
                $products_for_template[] = $presenter->present(
                    $presentationSettings,
                    $assembler->assembleProduct($rawProduct),
                    $this->context->language
                );
            }
        }
        return $products_for_template;
    }

    public function getBestSellers($carousel)
    {
        $nbProducts = $carousel['prod_display'];
        $searchProvider = new BestSalesProductSearchProvider(
            $this->context->getTranslator()
        );
        $context = new ProductSearchContext($this->context);
        $query = new ProductSearchQuery();
        $query->setResultsPerPage($nbProducts)->setPage(1);
        $query->setSortOrder(new SortOrder('product_sale', 'quantity', 'desc'));
        $result = $searchProvider->runQuery($context, $query);
        $assembler = new ProductAssembler($this->context);
        $presenterFactory = new ProductPresenterFactory($this->context);
        $presentationSettings = $presenterFactory->getPresentationSettings();
        $presenter = new ProductListingPresenter(
            new ImageRetriever(
                $this->context->link
            ),
            $this->context->link,
            new PriceFormatter(),
            new ProductColorsRetriever(),
            $this->context->getTranslator()
        );
        $products_for_template = [];
        $lpsbestsellersres = $result->getProducts();
        if ($carousel['random']) {
            shuffle($lpsbestsellersres);
        }
        foreach ($lpsbestsellersres as $rawProduct) {
            $products_for_template[] = $presenter->present(
                $presentationSettings,
                $assembler->assembleProduct($rawProduct),
                $this->context->language
            );
        }
        return $products_for_template;
    }

    public function getSpecials($carousel)
    {
        $lpsspecials = false;
        $nbProducts = $carousel['prod_display'];
        $lpsspecials = Product::getPricesDrop(
            (int) Context::getContext()->language->id,
            1,
            (int) $nbProducts,
            false,
            null,
            'ASC'
        );
        $assembler = new ProductAssembler($this->context);
        $presenterFactory = new ProductPresenterFactory($this->context);
        $presentationSettings = $presenterFactory->getPresentationSettings();
        $presenter = new ProductListingPresenter(
            new ImageRetriever(
                $this->context->link
            ),
            $this->context->link,
            new PriceFormatter(),
            new ProductColorsRetriever(),
            $this->context->getTranslator()
        );
        $products_for_template = [];
        if ($carousel['random']) {
            shuffle($lpsspecials);
        }
        if (is_array($lpsspecials)) {
            foreach ($lpsspecials as $rawProduct) {
                $products_for_template[] = $presenter->present(
                    $presentationSettings,
                    $assembler->assembleProduct($rawProduct),
                    $this->context->language
                );
            }
        }
        return $products_for_template;
    }
}
